#!/usr/bin/perl

# ARCADEmini converter: blm2ambin
# version 1.1 date 2003-09-12
# part of the ARCADEmini project - http[s]://arcademini.schuermans.info/
# written by 1stein <1stein@schuermans.info>
# based on blinkenplayer.pl by stray <stray@stray.ch>
# Copyleft: GNU public license - http://www.gnu.org/copyleft/gpl.html

print <<EOF;
ARCADEmini converter: blm2ambin
version 1.1 date 2003-09-12
part of the ARCADEmini project - http[s]://arcademini.schuermans.info/
written by 1stein <1stein\@schuermans.info>
based on blinkenplayer.pl by stray <stray\@stray.ch>
Copyleft: GNU public license - http://www.gnu.org/copyleft/gpl.html

EOF

if( @ARGV < 1 )
{
	print <<EOF;
syntax: blm2ambin <blm-file> [<ambin-file> [<amflash-file>]]

examples: blm2ambin example.blm
          blm2ambin example.blm example.ambin
          blm2ambin example.blm example.ambin example.amflash

EOF
	exit;
}

read_source( $ARGV[0] );

if( @ARGV < 2 ) { $file = $ARGV[0] . ".ambin"; } else { $file = $ARGV[1]; }
open( F, "> " . $file ) || die "Can't open output file \"$file\": $!\n";
binmode F;

if( @ARGV < 3 ) { $flash = 0; } else { $flash = 1; $file = $ARGV[2]; }
if( $flash ) { open( FF, "> " . $file ) || die "Can't open output file \"$file\": $!\n"; }

printf F "ARCADEmini 26x20/16 ambin 1.0   ";
for( $i = 32; $i < 512; $i++ ) { print F pack( 'C', 0xFF ); }
if( $flash )
{
	printf FF "amdata=EBFCEDEBEEEFGNGJGOGJCADCDGHIDCDACPDBDGCAGBGNGCGJGOCADBCODACACACA";
	for( $i = 32; $i < 512; $i++ ) { print FF "PP"; }
}

$char[0] = " ";
$char[1] = "#";

$bytes = 0;
$bytes_sector = 0;
$frames = 0;
$frames_sector;
$sectors = 0;
for( $f = 0; $f <= $#del; $f++) {

	$chars = @{@{$frames[$f]}[0]};
	if( $chars > 26 ) { $chars = 26 };
	$lines = @{$frames[$f]};
	if( $lines > 20 ) { $lines = 20 };
	$left = (26 - $chars) / 2;
	$right = 26 - $chars - $left;
	$top = (20 - $lines) / 2;
	$bottom = 20 - $lines - $top;

	$bit = 0;
	for( $i = 0; $i < 26 * $top; $i++, $bit++ ) { @bits[$bit] = 0; }
	$linenum = 0;
	foreach (@{$frames[$f]}) {
	  if( $linenum < 26 ) { for( $i = 0; $i < $left; $i++, $bit++ ) { @bits[$bit] = 0; } }
	  $charnum = 0;
	  foreach (@$_) {
		print($char[$_]);
		if( $linenum < 20 && $charnum < 26 ) {
		  @bits[$bit] = $_;
		  $bit++;
		}
		$charnum++;
	  }
	  if( $linenum < 26 ) { for( $i = 0; $i < $right; $i++, $bit++ ) { @bits[$bit] = 0; } }
	  print("\n");
	  $linenum++;
	}
	for( $i = 0; $i < 26 * $bottom; $i++, $bit++ ) { @bits[$bit] = 0; }

	print F pack( 'C', 0xAC );
	if( $flash ) { print FF to_flash( 0xAC ); }
	for( $byte = 0; $byte < 65; $byte++ ) {
		$val = 0;
		for( $bit = 0; $bit < 8; $bit++ ) {
			$val = $val * 2 + @bits[$byte*8+$bit];
		}
		print F pack( 'C', $val );
		if( $flash ) { print FF to_flash( $val ); }
	}
	print F pack( 'C', 0xC0 );
	if( $flash ) { print FF to_flash( 0xC0 ); }
	$bytes += 67;
	$bytes_sector += 67;

	print F pack( 'C', 0xC9 );
	if( $flash ) { print FF to_flash( 0xC9 ); }
	print F pack( 'C', ($del[$f] >> 8) & 0xFF );
	if( $flash ) { print FF to_flash( ($del[$f] >> 8) & 0xFF ); }
	print F pack( 'C', $del[$f] & 0xFF );
	if( $flash ) { print FF to_flash( $del[$f] & 0xFF ); }
	$bytes += 3;
	$bytes_sector += 3;

	$frames++;
	$frames_sector++;
	if( $frames_sector == 7 ) {
		for( $i = 0; $i < 22; $i++ ) { print F pack( 'C', 0 ); }
		if( $flash ) { for( $i = 0; $i < 22; $i++ ) { print FF to_flash( 0 ); } }
		$bytes += 22;
		$bytes_sector = 0;
		$frames_sector = 0;
		$sectors++;
	}
}

if( $bytes_sector > 0 )
{
	for( $i = 0; $i < 512 - $bytes_sector; $i++ ) { print F pack( 'C', 0 ); }
	if( $flash ) { for( $i = 0; $i < 512 - $bytes_sector; $i++ ) { print FF to_flash( 0 ); } }
	$bytes += 512 - $bytes_sector;
	$bytes_sector = 0;
	$frames_sector = 0;
	$sectors++;
}

$jump_sectors = 0xFFFFFFFF - ($sectors - 1);
print F pack( 'C', 0x01 );
print F pack( 'C', ($jump_sectors >> 24) & 0xFF );
print F pack( 'C', ($jump_sectors >> 16) & 0xFF );
print F pack( 'C', ($jump_sectors >> 8) & 0xFF );
print F pack( 'C', $jump_sectors & 0xFF );
for( $i = 5; $i < 512; $i++ ) { print F pack( 'C', 0x00 ); }
if( $flash )
{
	print FF to_flash( 0x01 );
	print FF to_flash( ($jump_sectors >> 24) & 0xFF );
	print FF to_flash( ($jump_sectors >> 16) & 0xFF );
	print FF to_flash( ($jump_sectors >> 8) & 0xFF );
	print FF to_flash( $jump_sectors & 0xFF );
	for( $i = 5; $i < 512; $i++ ) { print FF "AA"; }
}

if( $flash ) { print FF "&amloaded=1"; }

close F;
if( $flash ) { close FF; }

sub read_source {
  my $file = shift;
  open (F, $file) || die "Can't open input file \"$file\": $!\n";
  foreach $line (<F>) {
	chomp $line; chomp $line;
	$line =~ s/#.*//;
	next unless $line =~ /\S+/;
	if ($line =~ /^\@(\d+)/) {
	  push(@del,$1);
	  $frame = $#del;
	  next;
	}
	push(@{$frames[$frame]}, [split(undef,$line)]); 
  }
  close F;
}

sub to_flash
{
	my $val = shift;
	my $txt = pack( 'C', (($val >> 4) & 0x0F) + 0x41 ) . pack( 'C', ($val & 0x0F) + 0x41 );
	return $txt;
}

